"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FacadesBearerAsymmetricTokenVerifyProvider = void 0;
const tslib_1 = require("tslib");
// Copyright (c) 2023 Sourcefuse Technologies
//
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT
const context_1 = require("@loopback/context");
const repository_1 = require("@loopback/repository");
const rest_1 = require("@loopback/rest");
const jwt = tslib_1.__importStar(require("jsonwebtoken"));
const loopback4_authentication_1 = require("loopback4-authentication");
const moment_1 = tslib_1.__importDefault(require("moment"));
const jose = tslib_1.__importStar(require("node-jose"));
const repositories_1 = require("../../../repositories");
const logger_extension_1 = require("../../logger-extension");
const repositories_2 = require("../repositories");
let FacadesBearerAsymmetricTokenVerifyProvider = class FacadesBearerAsymmetricTokenVerifyProvider {
    constructor(revokedTokenRepository, logger, jwtKeysRepo, authUserModel) {
        this.revokedTokenRepository = revokedTokenRepository;
        this.logger = logger;
        this.jwtKeysRepo = jwtKeysRepo;
        this.authUserModel = authUserModel;
    }
    value() {
        return async (token, req) => {
            try {
                const isRevoked = await this.revokedTokenRepository.get(token);
                if (isRevoked === null || isRevoked === void 0 ? void 0 : isRevoked.token) {
                    throw new rest_1.HttpErrors.Unauthorized('TokenRevoked');
                }
            }
            catch (error) {
                if (rest_1.HttpErrors.HttpError.prototype.isPrototypeOf(error)) {
                    throw error;
                }
                this.logger.error('Revoked token repository not available !');
            }
            let user;
            try {
                // Get the key that matches the token's kid
                const decoded = jwt.decode(token.trim(), { complete: true });
                if (!decoded) {
                    throw new Error('Token is not valid');
                }
                const kid = decoded === null || decoded === void 0 ? void 0 : decoded.header.kid;
                // Load the JWKS
                const key = await this.jwtKeysRepo.findOne({
                    where: {
                        keyId: kid,
                    },
                });
                if (!key) {
                    throw new Error('Key not found for verification');
                }
                // Convert the JWK to PEM format for verification
                const jwkKey = await jose.JWK.asKey(key.publicKey, 'pem');
                const pem = jwkKey.toPEM(false);
                // Verify the token with the retrieved PEM-formatted public key
                user = jwt.verify(token, pem, {
                    issuer: process.env.JWT_ISSUER,
                    algorithms: ['RS256'],
                });
            }
            catch (error) {
                this.logger.error(JSON.stringify(error));
                throw new rest_1.HttpErrors.Unauthorized('TokenExpired');
            }
            if (user.passwordExpiryTime &&
                (0, moment_1.default)().isSameOrAfter((0, moment_1.default)(user.passwordExpiryTime))) {
                throw new rest_1.HttpErrors.Unauthorized('PasswordExpiryError');
            }
            if (this.authUserModel) {
                return new this.authUserModel(user);
            }
            else {
                return user;
            }
        };
    }
};
exports.FacadesBearerAsymmetricTokenVerifyProvider = FacadesBearerAsymmetricTokenVerifyProvider;
exports.FacadesBearerAsymmetricTokenVerifyProvider = FacadesBearerAsymmetricTokenVerifyProvider = tslib_1.__decorate([
    tslib_1.__param(0, (0, repository_1.repository)(repositories_2.RevokedTokenRepository)),
    tslib_1.__param(1, (0, context_1.inject)(logger_extension_1.LOGGER.LOGGER_INJECT)),
    tslib_1.__param(2, (0, repository_1.repository)(repositories_1.JwtKeysRepository)),
    tslib_1.__param(3, (0, context_1.inject)(loopback4_authentication_1.AuthenticationBindings.USER_MODEL, { optional: true })),
    tslib_1.__metadata("design:paramtypes", [repositories_2.RevokedTokenRepository, Object, repositories_1.JwtKeysRepository, Object])
], FacadesBearerAsymmetricTokenVerifyProvider);
//# sourceMappingURL=facades-bearer-asym-token-verify.provider.js.map